<?php

class ZynithSEOSitemap {

    public function __construct() {
        add_filter( 'wp_sitemaps_enabled', '__return_false' );
        add_action( 'template_redirect', [ $this, 'maybe_redirect_default_sitemap' ] );
        add_filter( 'cron_schedules', [ $this, 'add_schedules' ] );
        register_activation_hook( ZYNITH_SEO_FILE, [ $this, 'on_plugin_activation' ] );
        register_deactivation_hook( ZYNITH_SEO_FILE, [ $this, 'remove_event' ] );

        // Hooks and schedule names kept intact to prevent scheduler errors
        add_action( 'zynith_seo_generate_sitemap', [ $this, 'generate_sitemap' ] );
    }

    public function remove_event() {
    // Clear the scheduled event
    wp_clear_scheduled_hook( 'zynith_seo_generate_sitemap' );

    // Remove the sitemap file
    $sitemap_path = ZynithSEOSettings::get_sitemap_path();
    if ( file_exists( $sitemap_path ) ) {
        @unlink( $sitemap_path );
    }

    // Optionally, remove the sitemap stylesheet as well
    $stylesheet_path = ZynithSEOSettings::get_abspath() . 'sitemap.xsl';
    if ( file_exists( $stylesheet_path ) ) {
        @unlink( $stylesheet_path );
    }
}

    public static function init() {
        return new self();
    }

    public function on_plugin_activation() {
        $this->add_event();
        flush_rewrite_rules();
    }

    public function add_event() {
        if ( ! wp_next_scheduled( 'zynith_seo_generate_sitemap' ) ) {
            wp_schedule_event( time(), 'every_two_hours', 'zynith_seo_generate_sitemap' );
        }
    }

    public function add_schedules( $schedules ) {
		$custom_schedules = [ 'every_two_hours' => [
            'interval' => HOUR_IN_SECONDS * 2,
            'display' => __( 'Every Two Hours', ZYNITH_SEO_TEXT_DOMAIN )
        ] ];
		
        return array_merge( $schedules, $custom_schedules );
	}

    public function maybe_redirect_default_sitemap() {
        $sitemap_filename = ZynithSEOSettings::get_sitemap_name();
        if ( $sitemap_filename === 'wp-sitemap.xml' ) {
            return;
        }
        
        $current_url = $_SERVER['REQUEST_URI'];
        if ( strpos( $current_url, 'wp-sitemap.xml' ) !== false ) {
            wp_redirect( home_url( $sitemap_filename ), 301 );
            exit;
        }
    }

	private function update_sitemap_file() {
		$sitemap_path = ZynithSEOSettings::get_sitemap_path();
        $sitemap_filename = ZynithSEOSettings::get_sitemap_name();
        $last_sitemap_filename = get_option( 'last_sitemap_filename', null );

        if ( ! empty( $last_sitemap_filename ) ) {
            if ( $last_sitemap_filename !== $sitemap_filename ) {
                @unlink( ZynithSEOSettings::get_abspath() . $last_sitemap_filename );
            }
        }

        update_option( 'last_sitemap_filename', $sitemap_filename );

		@touch( $sitemap_path );
		@chmod( $sitemap_path, 0664 );

        $sitemap_content = $this->get_sitemap_xml();

        if ( wp_is_writable( $sitemap_path ) ) {
            if ( @file_put_contents( $sitemap_path, $sitemap_content ) !== false ) {
                return true;
            }
        }

		return false;
	}

	private function update_sitemap_stylesheet() {
		$stylesheet_path = ZynithSEOSettings::get_abspath() . 'sitemap.xsl';

		@touch( $stylesheet_path );
		@chmod( $stylesheet_path, 0664 );

        $stylesheet_content = $this->get_sitemap_stylesheet();

        if ( wp_is_writable( $stylesheet_path ) ) {
            if ( @file_put_contents( $stylesheet_path, $stylesheet_content ) !== false ) {
                return true;
            }
        }

		return false;
	}

    public function generate_sitemap_urls() {
        global $wpdb;

        $urls = [];
    
        // Generate URLs for public post types, categories, and authors
        $post_types = array_keys( ZynithSEOPostTypes::get_post_types() );
        $taxonomies = array_keys( ZynithSEOTaxonomies::get_taxonomies() );

        // Make sure authors are only added to sitemap if they have any content to show
        $authors = [];

        // Add checks for global noindex settings
        $noindex_post_types = get_option( 'zynith_noindex_post_types' );
        $noindex_taxonomies = get_option( 'zynith_noindex_taxonomies' );
        $noindex_archives = get_option( 'zynith_noindex_archives' );
        
        foreach ( $post_types as $post_type ) {
            if ( ! empty( $noindex_post_types ) && in_array( $post_type, array_keys( $noindex_post_types ) ) ) {
                continue;
            }
            
            // Switched to $wpdb as get_posts() results in infinite loop at this point
            $posts = $wpdb->get_results( $wpdb->prepare( "SELECT `ID`, `post_author` FROM `{$wpdb->base_prefix}posts` WHERE `post_type` = %s AND `post_status` = 'publish'", $post_type ) );

            if ( empty( $posts ) ) {
                continue;
            }
    
            foreach ( $posts as $post ) {
                $noindex = get_post_meta( $post->ID, '_custom_noindex', true );

                if ( $noindex === 'yes' || $noindex === true ) {
                    continue;
                }

                // If post is public, store the author ID
                $authors[ $post->post_author ] = $post->post_author;

                $urls[] = [
                    'loc' => htmlspecialchars( (string) get_permalink( $post->ID ) ),
                    'lastmod' => get_the_modified_date( 'Y-m-d\TH:i:sP', $post->ID ),
                ];
            }
        }
    
        foreach ( $taxonomies as $taxonomy ) {
            // Product shipping classes should always be excluded from sitemap
            if ( $taxonomy === 'product_shipping_class' || ( ! empty( $noindex_taxonomies ) && in_array( $taxonomy, array_keys( $noindex_taxonomies ) ) ) ) {
                continue;
            }

            $terms = get_terms( [ 'taxonomy'   => $taxonomy, 'hide_empty' => false, ] );
            if ( empty( $terms ) ) {
                continue;
            }
    
            foreach ( $terms as $term ) {
                $noindex = get_term_meta( $term->term_id, '_custom_noindex', true );
                if ( $noindex === 'yes' || $noindex === true ) {
                    continue;
                }

                $urls[] = [ 'loc' => htmlspecialchars( (string) get_term_link( $term->term_id, $taxonomy ) ), ];
            }
        }

        if ( ! empty( $noindex_archives ) && ! in_array( 'author', array_keys( $noindex_archives ) ) ) {
            if ( ! empty( $authors ) ) {
                foreach ( $authors as $author_id ) {
                    $noindex = get_user_meta( $author_id, '_custom_noindex', true );
                    if ( $noindex === 'yes' || $noindex === true ) {
                        continue;
                    }

                    $urls[] = [ 'loc' => htmlspecialchars( (string) get_author_posts_url( $author_id ) ), ];
                }
            }
        }

        return $urls;
    }

    public function get_sitemap_tags() {
        $urls = $this->generate_sitemap_urls();

        $xml_urls = '';
        if ( ! empty( $urls ) ) {
            foreach ( $urls as $url ) {
                $xml_urls .= '  <url>' . "\n";
                $xml_urls .= '    <loc>' . $url['loc'] . '</loc>' . "\n";

                if ( ! empty( $url['lastmod'] ) ) {
                    $xml_urls .= '    <lastmod>' . $url['lastmod'] . '</lastmod>' . "\n";
                }

                $xml_urls .= '  </url>' . "\n";
            }
        }

        return $xml_urls;
    }

    public function generate_sitemap() {
        $stylesheet_updated = $this->update_sitemap_stylesheet();
        $xml_updated = $this->update_sitemap_file();

        if ( ! $xml_updated ) {
            update_option( 'sitemap_xml_error', __( 'There was a problem saving your sitemap XML file. Please try to generate it manually from the settings.', ZYNITH_SEO_TEXT_DOMAIN ) );
        } elseif ( ! $stylesheet_updated ) {
            update_option( 'sitemap_xml_error', __( 'There was a problem saving your sitemap stylesheet. Please try to generate it manually from the settings.', ZYNITH_SEO_TEXT_DOMAIN ) );
        }

        // Just in case...
        flush_rewrite_rules();
    }

    public function get_sitemap_xml() {
        $xml_urls = $this->get_sitemap_tags();
        $stylesheet_path = ZynithSEOSettings::get_abspath() . 'sitemap.xsl';

        return '<?xml version="1.0" encoding="UTF-8"?>
        ' . ( file_exists( $stylesheet_path ) ? '<?xml-stylesheet type="text/xsl" href="' . home_url( 'sitemap.xsl' ) . '"?>' : '' ) . '
        <urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">
        <!-- Sitemap was generated on ' . date( 'Y-m-d H:i:s' ) . ' GMT -->
            ' . $xml_urls . '
        </urlset>';
    }

    public function get_sitemap_stylesheet() {
        $variables  = '<xsl:variable name="tableMinWidth" select="\'700\'"/>';
        $variables .= '<xsl:variable name="colorMain" select="\'#333333\'"/>';
        $variables .= '<xsl:variable name="colorAccent" select="\'#6002EE\'"/>';
        $variables .= '<xsl:variable name="colorAccentH1" select="\'#9a58fd\'"/>';
        $variables .= '<xsl:variable name="relativeFontColor" select="\'#ffffff\'"/>';

        $icon = '<link rel="icon" href="https://zynith.app/wp-content/uploads/zynith-seo-favicon.png" sizes="50x50"/>';
        $generator_link = '<a href="https://zynith.app/" target="_blank" rel="nofollow noopener">Zynith SEO</a>';

        return '<?xml version="1.0" encoding="UTF-8"?><xsl:stylesheet version="2.0" xmlns:sitemap="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"><xsl:output method="html" version="1.0" encoding="UTF-8" indent="yes"/><xsl:template match="/"><html xmlns="http://www.w3.org/1999/xhtml" lang="da-DK">
            <head><meta http-equiv="Content-Type" content="text/html; charset=utf-8"/><meta name="viewport" content="width=device-width, initial-scale=1"/>' . $variables . '<title>XML Sitemap &#x2d; ' . get_bloginfo( 'name' ) . '</title><style style="text/css">html{font-size: 62.5%;height: 100%;}body{font-size: 1.4rem;font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,Oxygen-Sans,Ubuntu,Cantarell,"Helvetica Neue",sans-serif;min-height: 100%;display: grid;grid-template-rows: auto 1fr auto;margin: 0;}.wrap{max-width: <xsl:value-of select="concat( $tableMinWidth, \'px\' )"/>;margin: 0 auto;overflow-wrap: break-word;}a{color: #3c68b9;text-decoration: none;}h1{font-size: 2.4rem;font-family: Verdana,Geneva,sans-serif;font-weight: normal;margin: 0;color: <xsl:value-of select="$colorAccentH1"/>;}h1 img{vertical-align: bottom;margin-right: 1.4rem;image-rendering: -webkit-optimize-contrast;}.rtl h1 img{margin-right: unset;margin-left: 1.4rem;}#description{background-color: <xsl:value-of select="$colorMain"/>;border-bottom: .7rem solid <xsl:value-of select="$colorAccent"/>;color: <xsl:value-of select="$relativeFontColor"/>;padding: 2rem 2rem 1.3rem;}#description a{color: <xsl:value-of select="$relativeFontColor"/>;}#content{padding: 2rem;background: #fff;}a:hover{border-bottom: 1px solid;}table{border-spacing: 0;table-layout: fixed;}th, td{font-size: 1.2rem;border: 0px solid;padding: 1rem 1.5rem;width: 100%;max-width: <xsl:value-of select="concat( $tableMinWidth - 159, \'px\' )"/>;min-width: 99px;overflow-wrap: anywhere;}th{text-align: left;border-bottom: 1px solid <xsl:value-of select="$colorAccent"/>;}.rtl th{text-align: right;}tr:nth-of-type(2n){background-color: #eaeaea;}#footer{padding: 0 3rem 2rem;font-size: 1.1rem;color: #999;}#footer a{color: inherit;}#description a, #footer a{border-bottom: 1px solid;}#description a:hover, #footer a:hover{border-bottom: none;}</style>' . $icon . '</head>
            <body class="ltr"><div id="description"><div class="wrap"><a href="' . home_url() . '"><h1>' . get_bloginfo( 'name' ) . ' — XML Sitemap</h1></a><p>This is a generated XML Sitemap, meant to be used by search engines like <a href="https://www.google.com/" target="_blank" rel="nofollow noreferrer noopener">Google</a> or <a href="https://www.bing.com/" target="_blank" rel="nofollow noreferrer noopener">Bing</a>.</p></div></div><div id="content"><div class="wrap"><table><thead><tr><th>URL</th><th>Last Updated</th></tr></thead><tbody><xsl:for-each select="sitemap:urlset/sitemap:url"><xsl:variable name="itemURL" select="sitemap:loc"/><xsl:variable name="lastmod" select="concat(substring(sitemap:lastmod,0,11),concat(\' \',substring(sitemap:lastmod,12,5)))"/><tr><td><a href="{$itemURL}"><xsl:choose><xsl:when test="string-length($itemURL)&gt;95"><xsl:value-of select="substring($itemURL,0,93)"/>...</xsl:when><xsl:otherwise><xsl:value-of select="$itemURL"/></xsl:otherwise></xsl:choose></a></td><td><xsl:value-of select="$lastmod"/></td></tr></xsl:for-each></tbody></table></div></div><div id="footer"><div class="wrap"><p>Generated by ' . $generator_link . '</p></div></div></body>
        </html></xsl:template></xsl:stylesheet>';
    }

}
